import java.net.*;
import java.io.*; 
                  
/**
  * A simple example for a java service.
  * This is a simple echoing Telnet Server.
  * It accepts telnet connections on port 23 and echos back every character typed.
*/                  
class TelnetEcho extends Thread implements Stopable
{                                               
  /**
    * The server cocket which accepts connections
  */
  ServerSocket ss;
                                                                 
  /**                                           
    * Here the telnet server implements the Stopable interface
    * On exit close the server cocket on port 23
  */
  public void onServiceStop()
  {                  
    try {
      if ( ss != null )
        ss.close();
    } catch (Exception e) {}
  }              
                                        
  /**
    * Don't wait if onServiceStop does not return. Terminate immediately.
  */                                        
  public int timeToWait()
  {
    return 0;
  }                                     

  /**
    * Open server socket and wait for incoming connections
  */                                                   
  public void run()
  {
    try {
      ss = new ServerSocket( 23 );
      new Echo( ss.accept() ).start();
      
    } catch (Exception e)
    {}
  }                  
  
  /**
    * The worker thread.
    * Get the socket input stream and echos bytes read
  */    
  class Echo extends Thread
  {              
    Socket s;
    Echo( Socket s )
    {
      this.s = s;
    }
    
    public void run()
    {
      try { 
        InputStream in = s.getInputStream();  
        OutputStream out = s.getOutputStream();  
                        
        while( true )
        {
          out.write( in.read() );
        }                
      } catch (Exception e)
      {}
    }
  }    
                                                                                                                            
  /**                                                         
    * The main method called when the service starts.
  */
  public static void main (String[] argv)
  {                                                  
    //Create echo server class
    TelnetEcho echo = new TelnetEcho();
    //Register it with the ServiceStopper
    ServiceStopper.stop( echo );         
    //Start the echo server thread
    echo.start();
  }
}               

